"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.grab = exports.tryJson = exports.GrabResponseError = void 0;
function getErrorMessage(e) {
    return (e === null || e === void 0 ? void 0 : e.message) || "unknown error";
}
// function getFetchImpl(): FetchFunction {
//     const fetchImpl = grab.__fetchImpl || globalThis.fetch;
//     if (!fetchImpl) {
//         throw new Error("No fetch implementation found. If you're using grab in node environment Please call grab.setup({ fetch })");
//     }
//     return fetchImpl;
// }
async function parseJsonResponse(result, method, url) {
    const text = await result.text();
    try {
        return JSON.parse(text);
    }
    catch (e) {
        const message = `Error parsing JSON (len=${text.length}) from ${method} ${url}: ${getErrorMessage(e)}`;
        console.log(`${message}. Full response:`, text);
        throw new Error(`${message}. See console for full response.`);
    }
}
class GrabResponseError extends Error {
    constructor(message, response, request) {
        super(message);
        this.response = response;
        this.request = request;
    }
}
exports.GrabResponseError = GrabResponseError;
function tryJson(res) {
    if (typeof res === "string") {
        try {
            return JSON.parse(res);
        }
        catch (e) {
            return res;
        }
    }
    return res;
}
exports.tryJson = tryJson;
const grabImpl = async (url, _a = {}) => {
    var _b;
    var { body } = _a, rest = __rest(_a, ["body"]);
    const urlWithQuery = ((_b = rest === null || rest === void 0 ? void 0 : rest.query) === null || _b === void 0 ? void 0 : _b.length) > 0 ? `${url}?${Object.entries(([k, v]) => `${k}=${encodeURIComponent(v)}`)}` : url;
    const method = rest.method || (body ? "POST" : "GET");
    let result;
    const requestParams = Object.assign({ method: method, body: body ? JSON.stringify(body) : undefined }, rest);
    try {
        result = await fetch(urlWithQuery, requestParams);
    }
    catch (e) {
        throw new Error(`Error calling ${method} ${url}: ${getErrorMessage(e)}`);
    }
    const getErrorText = async (result) => {
        try {
            return await result.text();
        }
        catch (e) {
            return "Unknown error";
        }
    };
    if (!result.ok) {
        let errorText = await getErrorText(result);
        const errorJson = tryJson(errorText);
        const message = `Error ${result.status} on ${method} ${url}`;
        console.log(message, errorJson);
        throw typeof errorJson === "string"
            ? new Error(`${message}: ${errorJson}`)
            : new GrabResponseError(message, errorJson, Object.assign(Object.assign({ url: urlWithQuery }, requestParams), { body: body || undefined }));
    }
    return await parseJsonResponse(result, method, url);
};
const grab = grabImpl;
exports.grab = grab;
grab.setup = (opts) => {
    grab.__fetchImpl = opts.fetch;
};
exports.default = grab;
